# load packages

library(lavaan)

# import data and name variables

wisc3miss <- read.table("~/Desktop/wisc3miss.dat", quote="\"")

names(wisc3miss) = c('id', 'verb1', 'verb2', 'verb4', 'verb6', 'perfo1', 'perfo2', 'perfo4', 'perfo6',
                     'info1', 'comp1', 'simi1', 'voca1', 'info6', 'comp6', 'simi6', 'voca6', 'momed', 'grad', 'constant')

# create missing data indicators and recode missing values

wisc3miss$perfo4r = 0
wisc3miss$perfo4r[wisc3miss$perfo4 == -99] <- 1
wisc3miss$perfo6r = 0
wisc3miss$perfo6r[wisc3miss$perfo6 == -99] <- 1

wisc3miss[wisc3miss==-99] <- NA

# correlate indicators with analysis variables
# see lavaan model below for fiml estimates of correlations

auxiliary <- wisc3miss[c('perfo1','perfo2','momed','grad')]
indicators <- wisc3miss[c('perfo4r','perfo6r')]
cor(auxiliary, indicators, use = "pairwise.complete.obs")

# ex1: identifying correlates of missingness

indicatorcorrs <- '
perfo1 ~~ perfo1
perfo2 ~~ perfo2
grad ~~ grad
momed ~~ momed
perfo4r ~~ perfo4r
perfo6r ~~ perfo6r
perfo1 ~~ perfo2 + grad + momed + perfo4r + perfo6r
perfo2 ~~ grad + momed + perfo4r + perfo6r
grad ~~ momed + perfo4r + perfo6r
momed ~~ perfo4r + perfo6r
perfo4r ~~ perfo6r
'

fitindicatorcorrs <- sem(indicatorcorrs, data = wisc3miss, meanstructure = TRUE, mimic = "mplus")
summary(fitindicatorcorrs, standardize = TRUE, fit.measures = TRUE)

# ex2: latent basis growth model with fiml

basis <- '
b0n =~ 1*perfo1 + 1*perfo2 + 1*perfo4 + 1*perfo6
b1n =~ 0*perfo1 + lo2*perfo2 + lo3*perfo4 + 1*perfo6
perfo1 ~~ res*perfo1
perfo2 ~~ res*perfo2
perfo4 ~~ res*perfo4
perfo6 ~~ res*perfo6
lo2 > 0
lo3 > 0
'

fitbasis <- growth(basis, data = wisc3miss, mimic = "mplus")
summary(fitbasis, fit.measures = TRUE)

# ex3: latent basis growth model with incomplete predictor

basispred <- '
grad ~ 1
grad ~~ grad
b0n =~ 1*perfo1 + 1*perfo2 + 1*perfo4 + 1*perfo6
b1n =~ 0*perfo1 + lo2*perfo2 + lo3*perfo4 + 1*perfo6
b0n ~ grad 
b1n ~ grad
perfo1 ~~ res*perfo1
perfo2 ~~ res*perfo2
perfo4 ~~ res*perfo4
perfo6 ~~ res*perfo6
lo2 > 0
lo3 > 0
'

fitbasispred <- growth(basispred, data = wisc3miss, mimic = "mplus", estimator = "mlr", fixed.x = FALSE)
summary(fitbasispred, fit.measures = TRUE)
inspect(fitbasispred, 'r2')

# ex4: latent basis growth model with manual auxiliary variable
# saturated correlates model has convergence issues
# link momed (aux) to grad (predictor) with regression (aux as extra outcome)

basisaux <- '
grad ~ 1
momed ~ 1
grad ~~ grad
momed ~~ momed
b0n =~ 1*perfo1 + 1*perfo2 + 1*perfo4 + 1*perfo6
b1n =~ 0*perfo1 + lo2*perfo2 + lo3*perfo4 + 1*perfo6
b0n ~ grad 
b1n ~ grad
momed ~ grad
perfo1 ~~ res*perfo1
perfo2 ~~ res*perfo2
perfo4 ~~ res*perfo4
perfo6 ~~ res*perfo6
momed ~~ perfo1 + perfo2 + perfo4 + perfo6
lo2 > 0
lo3 > 0
'

fitbasisaux <- growth(basisaux, data = wisc3miss, mimic = "mplus", estimator = "mlr", fixed.x = FALSE)
summary(fitbasisaux, fit.measures = TRUE)
inspect(fitbasisaux, 'r2')
